//
//  BMKLocHeadingController.m
//  BMKLocHeadingOCDemo
//
//  Created by v_chiyuanwei on 2020/7/15.
//  Copyright © 2020 baidu. All rights reserved.
//

#import "BMKLocHeadingController.h"

#import <BaiduMapAPI_Map/BMKMapComponent.h>
#import <BMKLocationkit/BMKLocationComponent.h>

@interface BMKLocHeadingController () <BMKLocationManagerDelegate>
/// 地图
@property (nonatomic, strong) BMKMapView *mapView;
/// 定位管理类
@property (nonatomic, strong) BMKLocationManager *locationManager;
/// 当前位置
@property (nonatomic, strong) BMKUserLocation *userLocation;

@end

@implementation BMKLocHeadingController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"定位图标指示方向";
    
    [self initialize];
    [self setupViews];
    [self customLocationView];
    [self startLocation];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // 当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [self.mapView viewWillAppear];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // 当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [self.mapView viewWillDisappear];
}

#pragma mark -
- (void)initialize {
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationController.navigationBar.translucent = NO;
    self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
    self.navigationController.navigationBar.barTintColor = [UIColor colorWithRed:34.f / 255.f green:37.f / 255.f blue:61.f / 255.f alpha:1.f];
    [self.navigationController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor whiteColor]}];
}

- (void)setupViews {
    [self.view addSubview:self.mapView];
}

/// 设置当前位置图标样式
- (void)customLocationView {
    BMKLocationViewDisplayParam *param = [[BMKLocationViewDisplayParam alloc] init];
    param.isAccuracyCircleShow = NO;
    param.locationViewHierarchy = LOCATION_VIEW_HIERARCHY_BOTTOM;
    param.locationViewImage = [UIImage imageNamed:@"user_location_heading"];
    [self.mapView updateLocationViewWithParam:param];
}

/// 开始定位
- (void)startLocation {
    // 开始连续定位
    [self.locationManager startUpdatingLocation];
    // 开始设备朝向事件回调
    [self.locationManager startUpdatingHeading];
}

#pragma mark - <BMKLocationManagerDelegate>
/// 连续定位回调函数
- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateLocation:(BMKLocation *)location orError:(NSError *)error {
    // 定位错误信息
    if (error) {
        NSLog(@"locError:{%zd - %@};", error.code, error.localizedDescription);
        return;
    }

    // 定位结果
    if (location) {
        self.userLocation.location = location.location;
        // 如需展示我的位置图标需要调用此方法
        [self.mapView updateLocationData:self.userLocation];
        // 设置地图中心点
        [self.mapView setCenterCoordinate:location.location.coordinate];
    }
}

/// 该方法为BMKLocationManager提供设备朝向的回调方法。
- (void)BMKLocationManager:(BMKLocationManager * _Nonnull)manager didUpdateHeading:(CLHeading * _Nullable)heading {
    if (!heading) { return; }
    
    /// 更新当前位置数据
    self.userLocation.heading = heading;
    [self.mapView updateLocationData:self.userLocation];
}

#pragma mark - getter
/// 地图
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, CONTENT_HEIGHT)];
        // 设定是否显示定位图层
        _mapView.showsUserLocation = YES;
        // 设定定位模式
        _mapView.userTrackingMode = BMKUserTrackingModeHeading;
    }
    return _mapView;
}

/// 定位管理类
- (BMKLocationManager *)locationManager {
    if (!_locationManager) {
        _locationManager = [[BMKLocationManager alloc] init];
        // 设置返回位置的坐标系类型
        _locationManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        // 设置距离过滤参数
        _locationManager.distanceFilter = kCLDistanceFilterNone;
        // 设置预期精度参数
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        // 设置应用位置类型
        _locationManager.activityType = CLActivityTypeAutomotiveNavigation;
        // 设置是否自动停止位置更新
        _locationManager.pausesLocationUpdatesAutomatically = NO;
        // 设置位置获取超时时间
        _locationManager.locationTimeout = 10;
        // 设置获取地址信息超时时间
        _locationManager.reGeocodeTimeout = 10;
        // 设置delegate
        _locationManager.delegate = self;
    }
    return _locationManager;
}

/// 当前位置
- (BMKUserLocation *)userLocation {
    if (!_userLocation) {
        _userLocation = [[BMKUserLocation alloc] init];
    }
    return _userLocation;
}

@end
